@@ -1093,7 +1093,7 @@ If CHOP-LENGTH is not specified, message set is not chopped."
       (save-match-data
 	(set-buffer send-buf)
 	(erase-buffer)
-	(elmo-set-buffer-multibyte nil)
+	(set-buffer-multibyte nil)
 	(if string
 	    (insert string)
 	  (with-current-buffer source-buf
@@ -2595,6 +2595,9 @@ If optional argument REMOVE is non-nil, remove FLAG."
 		      " () ")
 		    (elmo-imap4-buffer-literal send-buffer))))
 	  (kill-buffer send-buffer))
+	(when result
+	  (elmo-folder-preserve-flags
+	   folder (elmo-msgdb-get-message-id-from-buffer) flags))
 	result)
     ;; Unplugged
     (if elmo-enable-disconnected-operation
