@@ -393,7 +393,7 @@ file name for maildir directories."
     filename))
 
 (luna-define-method elmo-folder-append-buffer ((folder elmo-maildir-folder)
-					       &optional status number)
+					       &optional flags number)
   (let ((basedir (elmo-maildir-folder-directory-internal folder))
 	(src-buf (current-buffer))
 	dst-buf filename)
@@ -411,6 +411,8 @@ file name for maildir directories."
 	   (expand-file-name
 	    (concat "new/" (file-name-nondirectory filename))
 	    basedir))
+	  (elmo-folder-preserve-flags
+	   folder (elmo-msgdb-get-message-id-from-buffer) flags)
 	  t)
       ;; If an error occured, return nil.
       (error))))
@@ -449,7 +451,7 @@ file name for maildir directories."
   (if (elmo-folder-message-file-p src-folder)
       (let ((src-msgdb-exists (not (zerop (elmo-folder-length src-folder))))
 	    (dir (elmo-maildir-folder-directory-internal folder))
-	    (table (elmo-flag-table-load (elmo-folder-msgdb-path folder)))
+	    (table (elmo-folder-flag-table folder))
 	    (succeeds numbers)
 	    filename flags id)
 	(dolist (number numbers)
@@ -470,7 +472,7 @@ file name for maildir directories."
 	    (elmo-flag-table-set table id flags))
 	  (elmo-progress-notify 'elmo-folder-move-messages))
 	(when (elmo-folder-persistent-p folder)
-	  (elmo-flag-table-save (elmo-folder-msgdb-path folder) table))
+	  (elmo-folder-close-flag-table folder))
 	succeeds)
     (luna-call-next-method)))
 
