@@ -624,18 +624,7 @@ Don't cache if nil.")
 	    ret-val)))
 
 (defun elmo-nntp-make-msglist (beg-str end-str)
-  (elmo-set-work-buf
-   (let ((beg-num (string-to-int beg-str))
-	 (end-num (string-to-int end-str))
-	 i)
-     (setq i beg-num)
-     (insert "(")
-     (while (<= i end-num)
-       (insert (format "%s " i))
-       (setq i (1+ i)))
-     (insert ")")
-     (goto-char (point-min))
-     (read (current-buffer)))))
+  (elmo-make-number-list (string-to-int beg-str) (string-to-int end-str)))
 
 (luna-define-method elmo-folder-list-messages-plugged ((folder
 							elmo-nntp-folder)
@@ -936,7 +925,7 @@ Don't cache if nil.")
 	  ret-list ret-val beg)
       (set-buffer tmp-buffer)
       (erase-buffer)
-      (elmo-set-buffer-multibyte nil)
+      (set-buffer-multibyte nil)
       (insert string)
       (goto-char (point-min))
       (setq beg (point))
@@ -1389,7 +1378,7 @@ Returns a list of cons cells like (NUMBER . VALUE)"
   (save-excursion
     (let ((new-msgdb (elmo-make-msgdb))
 	  beg entity i num message-id)
-      (elmo-set-buffer-multibyte nil)
+      (set-buffer-multibyte nil)
       (goto-char (point-min))
       (setq i 0)
       (message "Creating msgdb...")
