@@ -64,11 +64,13 @@ See `shimbun-headers' for more detail about RANGE."
 			       (integer :tag "number"))))
   :group 'elmo)
 
-(defcustom elmo-shimbun-update-overview-folder-list nil
+(defcustom elmo-shimbun-update-overview-folder-list 'all
   "*List of FOLDER-REGEXP.
 FOLDER-REGEXP is the regexp of shimbun folder name which should be
-update overview when message is fetched."
-  :type '(repeat (regexp :tag "Folder Regexp"))
+update overview when message is fetched.
+If it is the symbol `all', update overview for all shimbun folders."
+  :type '(choice (const :tag "All shimbun folders" all)
+		 (repeat (regexp :tag "Folder Regexp")))
   :group 'elmo)
 
 ;; Shimbun header.
@@ -223,8 +225,11 @@ update overview when message is fetched."
       (when (nth 0 server-group) ; server
 	(elmo-shimbun-folder-set-shimbun-internal
 	 folder
-	 (shimbun-open (nth 0 server-group)
-		       (luna-make-entity 'shimbun-elmo-mua :folder folder))))
+	 (condition-case nil
+	     (shimbun-open (nth 0 server-group)
+			   (luna-make-entity 'shimbun-elmo-mua :folder folder))
+	   (file-error
+	    (luna-make-entity 'shimbun :server (nth 0 server-group))))))
       (when (nth 1 server-group)
 	(elmo-shimbun-folder-set-group-internal
 	 folder
@@ -427,9 +432,10 @@ update overview when message is fetched."
 	    shimbun-id)
 	(shimbun-article (elmo-shimbun-folder-shimbun-internal folder)
 			 header)
-	(when (elmo-string-match-member
-	       (elmo-folder-name-internal folder)
-	       elmo-shimbun-update-overview-folder-list)
+	(when (or (eq elmo-shimbun-update-overview-folder-list 'all)
+		  (elmo-string-match-member
+		   (elmo-folder-name-internal folder)
+		   elmo-shimbun-update-overview-folder-list))
 	  (elmo-shimbun-update-overview folder location header))
 	(when (setq shimbun-id
 		    (elmo-shimbun-header-extra-field header "x-shimbun-id"))
