@@ -36,6 +36,22 @@
 
 ;; Modified by Yamaoka not to use APEL functions.
 
+;; Unimplemented command:
+;;  @abbr
+;;  @float, @caption, @shortcaption, @listoffloats
+;;  @deftypecv[x]
+;;  @headitem
+;;  @comma{}
+;;  @quotation (optional arguments)
+;;  @acronym (optional argument)
+;;  @dofirstparagraphindent
+;;  @indent
+;;  @verbatiminclude
+;;  @\
+;;  @definfoenclose
+;;  @deftypeivar
+;;  @deftypeop
+
 ;;; Code:
 
 (require 'backquote)
@@ -200,6 +216,9 @@ DOCSTRING will be printed if ASSERTION is nil and
 (put 'page 'texinfo-format 'texinfo-discard-line)
 (put 'hyphenation 'texinfo-format 'texinfo-discard-command-and-arg)
 
+;; @slanted{} (makeinfo 4.8 or later)
+(put 'slanted 'texinfo-format 'texinfo-format-noop)
+
 ;; @tie{} (makeinfo 4.3 or later)
 (put 'tie 'texinfo-format 'texinfo-format-tie)
 (ptexinfmt-defun-if-void texinfo-format-tie ()
@@ -313,11 +332,13 @@ DOCSTRING will be printed if ASSERTION is nil and
 
 
 ;;; Marking
-;; @url, @env, @command
-(put 'url 'texinfo-format 'texinfo-format-code)
+;; @indicateurl, @url, @env, @command, 
 (put 'env 'texinfo-format 'texinfo-format-code)
 (put 'command 'texinfo-format 'texinfo-format-code)
 
+(put 'indicateurl 'texinfo-format 'texinfo-format-code)
+(put 'url 'texinfo-format 'texinfo-format-uref)	; Texinfo 4.7
+
 ;; @acronym
 (put 'acronym 'texinfo-format 'texinfo-format-var)
 
@@ -379,19 +400,36 @@ For example, @verb\{|@|\} results in @ and
   (delete-char 1))
 
 
-;;; Accents and Special characters
-;; @registeredsymbol{}	==>	(R)
-(put 'pounds 'texinfo-format 'texinfo-format-registeredsymbol)
+;;; @LaTeX, @registeredsymbol{}
+(put 'LaTeX 'texinfo-format 'texinfo-format-LaTeX)
+(ptexinfmt-defun-if-void texinfo-format-LaTeX ()
+  (texinfo-parse-arg-discard)
+  (insert "LaTeX"))
+
+(put 'registeredsymbol 'texinfo-format 'texinfo-format-registeredsymbol)
 (ptexinfmt-defun-if-void texinfo-format-registeredsymbol ()
   (texinfo-parse-arg-discard)
   (insert "(R)"))
 
+;;; Accents and Special characters
 ;; @pounds{}	==>	#	Pounds Sterling
 (put 'pounds 'texinfo-format 'texinfo-format-pounds)
 (ptexinfmt-defun-if-void texinfo-format-pounds ()
   (texinfo-parse-arg-discard)
   (insert "#"))
 
+;; @ordf{}	==>	a	Spanish feminine
+(put 'ordf 'texinfo-format 'texinfo-format-ordf)
+(ptexinfmt-defun-if-void texinfo-format-ordf ()
+  (texinfo-parse-arg-discard)
+  (insert "o"))
+
+;; @ordm{}	==>	o	Spanish masculine
+(put 'ordm 'texinfo-format 'texinfo-format-ordm)
+(ptexinfmt-defun-if-void texinfo-format-ordm ()
+  (texinfo-parse-arg-discard)
+  (insert "o"))
+
 ;; @OE{}	==>	OE	French-OE-ligature
 (put 'OE 'texinfo-format 'texinfo-format-French-OE-ligature)
 (ptexinfmt-defun-if-void texinfo-format-French-OE-ligature ()
