@@ -124,6 +124,7 @@ Return number if put mark succeed"
 
 (defun wl-summary-target-mark-set-action (action)
   (unless (eq (wl-summary-action-symbol action) 'target-mark)
+    (unless wl-summary-buffer-target-mark-list (error "no target"))
     (save-excursion
       (goto-char (point-min))
       (let ((numlist wl-summary-buffer-number-list)
@@ -662,12 +663,12 @@ Return number if put mark succeed"
 			     (wl-summary-action-mark action)
 			     nil) collected)))
     (dolist (mark-info wl-summary-buffer-temp-mark-list)
-      (if numbers
-	  (when (memq (nth 0 mark-info) numbers)
-	    (setq pair (assoc (nth 1 mark-info) collected)))
-	(setq pair (assoc (nth 1 mark-info) collected)))
-      (setq pair (assoc (nth 1 mark-info) collected))
-      (setcdr pair (cons mark-info (cdr pair))))
+      (setq pair
+	    (when (or (null numbers)
+		      (memq (nth 0 mark-info) numbers))
+	      (assoc (nth 1 mark-info) collected)))
+      (if pair
+	  (setcdr pair (cons mark-info (cdr pair)))))
     ;; collected is a pair of
     ;; mark-string and a list of mark-info
     (dolist (pair collected)
@@ -689,7 +690,7 @@ Return number if put mark succeed"
       (setq wl-message-buffer nil))
     (set-buffer-modified-p nil)
     (when (> failures 0)
-      (format "%d execution(s) were failed" failures))))
+      (message "%d execution(s) were failed" failures))))
 
 (defun wl-summary-exec-region (beg end)
   (interactive "r")
@@ -930,10 +931,7 @@ If optional argument NUMBER is specified, unmark message specified by NUMBER."
 
 (defun wl-summary-target-mark-all ()
   (interactive)
-  (wl-summary-target-mark-region (point-min) (point-max))
-  (setq wl-summary-buffer-target-mark-list
-	(elmo-folder-list-messages wl-summary-buffer-elmo-folder
-				   t 'in-msgdb)))
+  (wl-summary-target-mark-region (point-min) (point-max)))
 
 (defun wl-summary-delete-all-mark (mark)
   (goto-char (point-min))
