@@ -36,6 +36,7 @@
 (require 'emu)
 (condition-case nil (require 'timezone) (error nil))
 (require 'std11)
+(require 'eword-encode)
 (require 'wl-vars)
 
 (defvar x-face-add-x-face-version-header)
@@ -43,6 +44,7 @@
 (defvar mail-from-style)
 
 (eval-when-compile
+  (require 'cl)
   (require 'static)
   (require 'elmo-pop3)
   (defalias-maybe 'x-face-insert 'ignore)
@@ -224,50 +226,56 @@ e.g.
 		   (list (concat wl-draft-mime-bcc-field-name  ":")))))))
 
 (defun wl-draft-make-mail-followup-to (recipients)
-  (if (elmo-list-member
-       (or wl-user-mail-address-list
-	   (list (wl-address-header-extract-address wl-from)))
-       recipients)
-      (let ((rlist (elmo-list-delete
-		    (or wl-user-mail-address-list
-			(list (wl-address-header-extract-address wl-from)))
-		    recipients
-		    (lambda (elem list)
-		      (elmo-delete-if
-		       (lambda (item) (string= (downcase elem)
-					       (downcase item)))
-		       list)))))
-	(if (elmo-list-member rlist (mapcar 'downcase
-					    wl-subscribed-mailing-list))
-	    rlist
-	  (append rlist (list (wl-address-header-extract-address
-			       wl-from)))))
-    recipients))
+  (let ((rlist (wl-address-delete-user-mail-addresses recipients)))
+    (if (elmo-list-member rlist (mapcar 'downcase
+					wl-subscribed-mailing-list))
+	rlist
+      (append rlist (list (wl-address-header-extract-address
+			   wl-from))))))
 
 (defun wl-draft-delete-myself-from-cc (to cc)
-  (let ((myself (or wl-user-mail-address-list
-		    (list (wl-address-header-extract-address wl-from)))))
-    (cond (wl-draft-always-delete-myself ; always-delete option
-	   (elmo-list-delete myself cc
-			     (lambda (elem list)
-			       (elmo-delete-if
-				(lambda (item) (string= (downcase elem)
-							(downcase item)))
-				list))))
-	  ((elmo-list-member (append to cc) ; subscribed mailing-list
-			     (mapcar 'downcase wl-subscribed-mailing-list))
-	   (elmo-list-delete myself cc
-			     (lambda (elem list)
-			       (elmo-delete-if
-				(lambda (item) (string= (downcase elem)
-							(downcase item)))
-				list))))
-	  (t cc))))
+  (cond (wl-draft-always-delete-myself ; always-delete option
+	 (wl-address-delete-user-mail-addresses cc))
+	((elmo-list-member (append to cc) ; subscribed mailing-list
+			   (mapcar 'downcase wl-subscribed-mailing-list))
+	 (wl-address-delete-user-mail-addresses cc))
+	(t cc)))
+
+(defsubst wl-draft-strip-subject-regexp (subject regexp)
+  "Remove REGEXP from SUBJECT string."
+  (if (string-match regexp subject)
+      (substring subject (match-end 0))
+    subject))
+
+(defun wl-draft-forward-make-subject (original-subject)
+  "Generate subject string for forwarding."
+  (cond ((functionp wl-forward-subject-prefix)
+	 (concat (funcall wl-forward-subject-prefix)
+		 original-subject))
+	((stringp wl-forward-subject-prefix)
+	 (concat wl-forward-subject-prefix
+		 (wl-draft-strip-subject-regexp
+		  (or original-subject "")
+		  wl-subject-forward-prefix-regexp)))
+	(t original-subject)))
+
+(defun wl-draft-reply-make-subject (original-subject)
+  "Generate subject string for replying."
+  (cond ((functionp wl-reply-subject-prefix)
+	 (concat (funcall wl-reply-subject-prefix)
+		 original-subject))
+	((stringp wl-reply-subject-prefix)
+	 (concat wl-reply-subject-prefix
+		 (wl-draft-strip-subject-regexp
+		  (or original-subject "")
+		  wl-subject-re-prefix-regexp)))
+	(t original-subject)))
 
 (defun wl-draft-forward (original-subject summary-buf)
-  (let (references parent-folder)
+  (let (references parent-folder subject)
     (with-current-buffer summary-buf
       (setq parent-folder (wl-summary-buffer-folder-name)))
+    (setq subject (wl-draft-forward-make-subject original-subject))
     (with-current-buffer (wl-message-get-original-buffer)
       (setq references (nconc
 			(std11-field-bodies '("References" "In-Reply-To"))
@@ -282,20 +290,13 @@ e.g.
 	 (get-buffer-window summary-buf)
 	 (select-window (get-buffer-window summary-buf)))
     (wl-draft (list (cons 'To "")
-		    (cons 'Subject
-			  (concat wl-forward-subject-prefix original-subject))
+		    (cons 'Subject subject)
 		    (cons 'References references))
 	      nil nil nil nil parent-folder))
   (goto-char (point-max))
   (wl-draft-insert-message)
   (mail-position-on-field "To"))
 
-(defun wl-draft-strip-subject-re (subject)
-  "Remove \"Re:\" from SUBJECT string. Shamelessly copied from Gnus."
-  (if (string-match wl-subject-prefix-regexp subject)
-      (substring subject (match-end 0))
-    subject))
-
 (defun wl-draft-self-reply-p ()
   "Return t when From address in the current message is user's self one or not."
   (wl-address-user-mail-address-p (or (elmo-field-body "From") "")))
@@ -368,10 +369,11 @@ Reply to author if WITH-ARG is non-nil."
     (setq to (wl-parse-addresses to)
 	  cc (wl-parse-addresses cc))
     (with-temp-buffer			; to keep raw buffer unibyte.
-      (elmo-set-buffer-multibyte default-enable-multibyte-characters)
+      (set-buffer-multibyte default-enable-multibyte-characters)
       (setq decoder (mime-find-field-decoder 'Subject 'plain))
-      (setq subject (if (and subject decoder)
-			(funcall decoder subject) subject))
+      (setq subject (wl-draft-reply-make-subject
+		     (if (and subject decoder)
+			 (funcall decoder subject) subject)))
       (setq to-alist
 	    (mapcar
 	     (lambda (addr)
@@ -386,10 +388,6 @@ Reply to author if WITH-ARG is non-nil."
 	       (cons (nth 1 (std11-extract-address-components addr))
 		     (if decoder (funcall decoder addr) addr)))
 	     cc)))
-    (and wl-reply-subject-prefix
-	 (setq subject (concat wl-reply-subject-prefix
-			       (wl-draft-strip-subject-re
-				(or subject "")))))
     (setq in-reply-to (std11-field-body "Message-Id"))
     (setq references (nconc
 		      (std11-field-bodies '("References" "In-Reply-To"))
@@ -405,7 +403,7 @@ Reply to author if WITH-ARG is non-nil."
 	    (wl-draft-make-mail-followup-to (append to cc)))
       (setq mail-followup-to (wl-delete-duplicates mail-followup-to nil t)))
     (with-temp-buffer			; to keep raw buffer unibyte.
-      (elmo-set-buffer-multibyte default-enable-multibyte-characters)
+      (set-buffer-multibyte default-enable-multibyte-characters)
       (setq newsgroups (wl-parse newsgroups
 				 "[ \t\f\r\n,]*\\([^ \t\f\r\n,]+\\)")
 	    newsgroups (wl-delete-duplicates newsgroups)
@@ -516,12 +514,13 @@ Reply to author if WITH-ARG is non-nil."
   (save-restriction
     (narrow-to-region (point)(point))
     (insert
-     (with-current-buffer mail-reply-buffer
-       (when decode-it
-	 (decode-mime-charset-region (point-min) (point-max)
-				     wl-mime-charset))
-       (buffer-substring-no-properties
-	(point-min) (point-max))))
+     (string-as-multibyte
+      (with-current-buffer mail-reply-buffer
+	(when decode-it
+	  (decode-mime-charset-region (point-min) (point-max)
+				      wl-mime-charset))
+	(buffer-substring-no-properties
+	 (point-min) (point-max)))))
     (when ignored-fields
       (goto-char (point-min))
       (wl-draft-delete-fields ignored-fields))
@@ -608,10 +607,7 @@ Reply to author if WITH-ARG is non-nil."
 	(set-buffer
 	 (wl-draft (list
 		    (cons 'From
-			  (if (member
-			       (nth 1 (std11-extract-address-components from))
-			       wl-user-mail-address-list)
-			      from))
+			  (if (wl-address-user-mail-address-p from) from))
 		    (cons 'To to)
 		    (cons 'Cc cc)
 		    (cons 'Subject subject)
@@ -636,17 +632,22 @@ Reply to author if WITH-ARG is non-nil."
 	mail-citation-hook mail-yank-hooks
 	wl-draft-add-references wl-draft-add-in-reply-to
 	wl-draft-cite-function)
-    (with-current-buffer wl-draft-buffer-cur-summary-buffer
-      (with-current-buffer wl-message-buffer
-	(setq original-buffer (wl-message-get-original-buffer))
-	(if (zerop
-	     (with-current-buffer original-buffer
-	       (buffer-size)))
-	    (error "No current message"))))
-    (setq mail-reply-buffer original-buffer)
-    (wl-draft-yank-from-mail-reply-buffer
-     nil
-     wl-ignored-forwarded-headers)))
+    (if (and wl-draft-buffer-cur-summary-buffer
+	     (with-current-buffer wl-draft-buffer-cur-summary-buffer
+	       (and wl-message-buffer
+		    (with-current-buffer wl-message-buffer
+		      (setq original-buffer (wl-message-get-original-buffer))
+		      (not (zerop (with-current-buffer original-buffer
+				    (buffer-size))))))))
+	(progn
+	  (setq mail-reply-buffer original-buffer)
+	  (wl-draft-yank-from-mail-reply-buffer
+	   nil
+	   wl-ignored-forwarded-headers))
+      (when (string= (mime-make-tag "message" "rfc822")
+		     (buffer-substring-no-properties (point-at-bol 0)(point-at-eol 0)))
+	(delete-region (point-at-bol 0) (1+ (point-at-eol 0))))
+      (error "No current message"))))
 
 (defun wl-draft-insert-get-message (dummy)
   (let ((fld (completing-read
@@ -1115,7 +1116,7 @@ non-nil."
 (defun wl-draft-send-mail-with-pop-before-smtp ()
   "Send the prepared message buffer with POP-before-SMTP."
   (require 'elmo-pop3)
-  (let ((session
+  (let ((folder
 	 (luna-make-entity
 	  'elmo-pop3-folder
 	  :user   (or wl-pop-before-smtp-user
@@ -1126,16 +1127,17 @@ non-nil."
 		      elmo-pop3-default-port)
 	  :auth   (or wl-pop-before-smtp-authenticate-type
 		      elmo-pop3-default-authenticate-type)
-	  :stream-type (or wl-pop-before-smtp-stream-type
-			   elmo-pop3-default-stream-type))))
+	  :stream-type (elmo-get-network-stream-type
+			(or wl-pop-before-smtp-stream-type
+			    elmo-pop3-default-stream-type))))
+	session)
     (condition-case error
 	(progn
-	  (elmo-pop3-get-session session)
+	  (setq session (elmo-pop3-get-session folder))
 	  (when session (elmo-network-close-session session)))
       (error
-       (elmo-network-close-session session)
        (unless (string= (nth 1 error) "Unplugged")
-	 (signal (car error)(cdr error))))))
+	 (signal (car error) (cdr error))))))
   (wl-draft-send-mail-with-smtp))
 
 (defun wl-draft-insert-required-fields (&optional force-msgid)
@@ -1443,7 +1445,11 @@ If KILL-WHEN-DONE is non-nil, current draft buffer is killed"
 		(insert (if (eq (char-before) ?\n) "" "\n")
 			mail-header-separator "\n")))
 	    (let ((mime-header-encode-method-alist
-		   '((eword-encode-unstructured-field-body))))
+		   (append
+		    '((eword-encode-unstructured-field-body
+		       .  (To Cc Bcc Resent-To Resent-Cc Resent-Bcc From)))
+		    (if (boundp 'mime-header-encode-method-alist)
+			(symbol-value 'mime-header-encode-method-alist)))))
 	      (mime-edit-translate-buffer))
 	    (wl-draft-get-header-delimiter t)
 	    (setq next-number
@@ -1731,6 +1737,7 @@ If KILL-WHEN-DONE is non-nil, current draft buffer is killed"
 	       (error "Invalid value for wl-draft-buffer-style"))))))
     (auto-save-mode -1)
     (wl-draft-mode)
+    (set-buffer-multibyte t)		; draft buffer is always multibyte.
     (make-local-variable 'truncate-partial-width-windows)
     (setq truncate-partial-width-windows nil)
     (setq truncate-lines wl-draft-truncate-lines)
@@ -2196,14 +2203,14 @@ Automatically applied in draft sending time."
 	      (setq found t)))
 	    (if (and found wl-draft-config-matchone)
 		(throw 'done t))
-	    (setq alist (cdr alist))))
-	(if found
-	    (setq wl-draft-config-exec-flag nil))
-	(run-hooks 'wl-draft-config-exec-hook)
-	(put-text-property (point-min)(point-max) 'face nil)
-	(wl-highlight-message (point-min)(point-max) t)
-	(setq wl-draft-config-variables
-	      (elmo-uniq-list local-variables))))))
+	    (setq alist (cdr alist)))))
+      (if found
+	  (setq wl-draft-config-exec-flag nil))
+      (run-hooks 'wl-draft-config-exec-hook)
+      (put-text-property (point-min)(point-max) 'face nil)
+      (wl-highlight-message (point-min)(point-max) t)
+      (setq wl-draft-config-variables
+	    (elmo-uniq-list local-variables)))))
 
 (defun wl-draft-replace-field (field content &optional add)
   (save-excursion
