@@ -118,7 +118,7 @@ With association ((\"folder\" message \"message-id\") . cache-buffer).")
   "A major mode for original message buffer."
   (setq major-mode 'wl-original-message-mode)
   (setq buffer-read-only t)
-  (elmo-set-buffer-multibyte nil)
+  (set-buffer-multibyte nil)
   (setq mode-name "Wanderlust original message"))
 
 (defun wl-original-message-buffer-get (name)
@@ -338,15 +338,18 @@ Returns non-nil if bottom of message."
   (setq mode-name "Message"))
 
 (defun wl-message-exit ()
-  "Move to summary buffer."
+  "Move to summary buffer or mother buffer."
   (interactive)
-  (let (summary-buf summary-win)
-    (if (setq summary-buf wl-message-buffer-cur-summary-buffer)
-	(if (setq summary-win (get-buffer-window summary-buf))
-	    (select-window summary-win)
-	  (switch-to-buffer summary-buf)
-	  (wl-message-select-buffer wl-message-buffer)
-	  (select-window (get-buffer-window summary-buf))))
+  (let (summary-buf summary-win mother-buffer)
+    (cond ((setq summary-buf wl-message-buffer-cur-summary-buffer)
+	   (if (setq summary-win (get-buffer-window summary-buf))
+	       (select-window summary-win)
+	     (switch-to-buffer summary-buf)
+	     (wl-message-select-buffer wl-message-buffer)
+	     (select-window (get-buffer-window summary-buf))))
+	  ((setq mother-buffer mime-mother-buffer)
+	   (kill-buffer (current-buffer))
+	   (switch-to-buffer mother-buffer)))
     (run-hooks 'wl-message-exit-hook)))
 
 (defun wl-message-toggle-disp-summary ()
