@@ -144,11 +144,20 @@
   "*A list of user's mail addresses.
 This list is used to judge whether an address is user's or not.
 You should set this variable if you use multiple e-mail addresses.
-If you don't have multiple e-mail addresses, you don't have to set this."
+If you don't have multiple e-mail addresses, you don't have to set this.
+NOTE: Non-nil value of `wl-user-mail-address-regexp' supersede this."
   :type '(repeat string)
   :group 'wl
   :group 'wl-setting)
 
+(defcustom wl-user-mail-address-regexp nil
+  "*A regexp for user's mail addresses.
+Supersede `wl-user-mail-address-list'."
+  :type '(choice (const :tag "Use wl-user-mail-address-list" nil)
+		 string)
+  :group 'wl
+  :group 'wl-setting)
+
 (defcustom wl-organization (getenv "ORGANIZATION")
   "Organization name."
   :type '(choice (const :tag "none" nil)
@@ -406,12 +415,14 @@ EXEC-FUNCTION is a function called to execute the action.
 Its argument is a list of MARK-INFO.
 MARK-INFO is a list of (NUMBER MARK DATA).
 FACE is a face for highlighting."
-  :type '(repeat (string :tag "Temporary mark")
-		 (symbol :tag "Set mark function")
-		 (symbol :tag "Unset mark function")
-		 (symbol :tag "Exec function")
-		 (symbol :tag "Face symbol")
-		 (string :tag "Document string"))
+  :type '(repeat (list
+		  (string :tag "Temporary mark")
+		  (symbol :tag "Action name")
+		  (symbol :tag "Argument function")
+		  (symbol :tag "Set mark function")
+		  (symbol :tag "Exec function")
+		  (symbol :tag "Face symbol")
+		  (string :tag "Document string")))
   :group 'wl-summary)
 
 ;; Important folders
@@ -572,6 +583,7 @@ If nil, don't authenticate."
   :type '(choice (const :tag "none" nil)
 		 (const :tag "PLAIN" "plain")
 		 (const :tag "CRAM-MD5" "cram-md5")
+		 (const :tag "DIGEST-MD5" "digest-md5")
 		 (const :tag "LOGIN" "login")
 		 (string :tag "Other"))
   :group 'wl
@@ -849,14 +861,21 @@ Prepared candidates are 'wl-draft-send-mail-with-smtp,
 	       ("Mail-Followup-To" "To" "Cc")
 	       ("Followup-To" "Newsgroups"))))
   "Alist of cons cell of
-\('field-name' .  ('fields for To' 'fields for Cc' 'fields for Newsgroups'))
-'field-name' is a string.
+\('condition' .  ('fields for To' 'fields for Cc' 'fields for Newsgroups'))
+'condition' is a header name string (non-nil if the header exists in original
+message), a function (evaluated in original message buffer) or a list of those
+\(means 'AND' condition).
 'fields for ***' is a list of strings.
-If car of each cons cell exists in original message,
-cdr of each cons cell is used for draft message.
+If car of each cons cell returns non-nil value,
+cdr of each cons cell is used for preparing headers of draft message.
 Default is for 'reply-to-all'."
   :type '(repeat (cons (choice (string :tag "Field Name")
-			       (repeat (string :tag "Field Name")))
+			       (symbol :tag "Function")
+			       (const :tag "Replying to self" wl-draft-self-reply-p)
+			       (repeat :tag "AND"
+				       (choice (string :tag "Field Name")
+					       (symbol :tag "Function")
+					       (const :tag "Replying to self" wl-draft-self-reply-p))))
 		       (list (repeat :tag "Fields For To" string)
 			     (repeat :tag "Fields For Cc" string)
 			     (repeat :tag "Fields For Newsgroups" string))))
@@ -871,13 +890,20 @@ Default is for 'reply-to-all'."
     (wl-draft-self-reply-p . (("To") ("Cc") nil))
     ("From" . (("From") ("To" "Cc") nil)))
   "Alist of cons cell of
-\('field-name' .  ('fields for To' 'fields for Cc' 'fields for Newsgroups'))
-'field-name' is a string.
+\('condition' .  ('fields for To' 'fields for Cc' 'fields for Newsgroups'))
+'condition' is a header name string (non-nil if the header exists in original
+message), a function (evaluated in original message buffer) or a list of those
+\(means 'AND' condition).
 'fields for ***' is a list of strings.
-If car of each cons cell exists in original message,
-cdr of each cons cell is used for draft message."
+If car of each cons cell returns non-nil value,
+cdr of each cons cell is used for preparing headers of draft message."
   :type '(repeat (cons (choice (string :tag "Field Name")
-			       (repeat (string :tag "Field Name")))
+			       (symbol :tag "Function")
+			       (const :tag "Replying to self" wl-draft-self-reply-p)
+			       (repeat :tag "AND"
+				       (choice (string :tag "Field Name")
+					       (symbol :tag "Function")
+					       (const :tag "Replying to self" wl-draft-self-reply-p))))
 		       (list (repeat :tag "Fields For To" string)
 			     (repeat :tag "Fields For Cc" string)
 			     (repeat :tag "Fields For Newsgroups" string))))
@@ -987,14 +1013,16 @@ cdr of each cons cell is used for draft message."
   '((important "orange"))
   "An alist to define the flags for the summary mode.
 Each element is a form like:
-\(SYMBOL-OF-FLAG COLOR\)
+\(SYMBOL-OF-FLAG COLOR [MARK]\)
 Example:
 \((important \"orange\"\)
- \(todo \"red\"\)
- \(business \"green\"\)
+ \(todo \"red\" \"T\"\)
+ \(business \"green\" \"B\"\)
  \(private \"blue\"\)\)"
   :type '(repeat (list (symbol :tag "flag")
-		       (string :tag "color")))
+		       (string :tag "color")
+		       (choice (string :tag "mark")
+			       (const :tag "Default mark" nil))))
   :group 'wl-summary)
 
 (defcustom wl-summary-new-uncached-mark "N"
@@ -1278,7 +1306,10 @@ See also variable `wl-draft-parent-folder'."
   :group 'wl-draft)
 
 (defcustom wl-template-alist nil
-  "Alist of template."
+  "Alist of template.
+First element of each list is a string specifies the name of the template.
+Remaining elements indicate actions. The format of actions is same as that
+of `wl-draft-config-alist'."
   :type '(repeat (list (string :tag "Name")
 		       (repeat
 			:inline t
@@ -1404,7 +1435,8 @@ You can also set it to a list of setting.
 
 (defcustom wl-draft-sendlog-max-size 20000
   "*Max file size of sendlog."
-  :type 'integer
+  :type '(choice (const :tag "Unlimited" nil)
+		 integer)
   :group 'wl-draft)
 
 (defcustom wl-summary-default-number-column 5
@@ -1710,14 +1742,18 @@ This variable overwhelm `wl-message-ignored-field-list' settings."
   :group 'wl-pref)
 
 (defcustom wl-reply-subject-prefix "Re: "
-  "*Prefix of the subject of the replied message."
-  :type 'string
+  "*Prefix of the subject of the replied message.
+The value is string or string valued function to be evalueted in the target
+message buffer."
+  :type '(choice string function)
   :group 'wl-draft
   :group 'wl-pref)
 
 (defcustom wl-forward-subject-prefix "Forward: "
-  "*Prefix of the subject of the forwarded message."
-  :type 'string
+  "*Prefix of the subject of the forwarded message.
+The value is string or string valued function to be evalueted in the target
+message buffer."
+  :type '(choice string function)
   :group 'wl-draft
   :group 'wl-pref)
 
@@ -1727,12 +1763,18 @@ This variable overwhelm `wl-message-ignored-field-list' settings."
   :group 'wl-pref
   :group 'wl-draft)
 
-(defcustom wl-subject-prefix-regexp "^[ \t]*\\([Rr][Ee][:>][ \t]*\\)*[ \t]*"
+(defcustom wl-subject-re-prefix-regexp "^[ \t]*\\([Rr][Ee][:>][ \t]*\\)*[ \t]*"
   "*Regexp matching \"Re: \" in the subject line."
   :type 'regexp
   :group 'wl-draft
   :group 'wl-pref)
 
+(defcustom wl-subject-forward-prefix-regexp "^[ \t]*\\(\\([Ff][Oo][Rr][Ww][Aa][Rr][Dd]\\|[Ff][Ww][Dd]\\|[Ff][Ww]\\)[:>][ \t]*\\)*[ \t]*"
+  "*Regexp matching \"Forward: \", \"Fwd: \", or \"Fw: \" in the subject line."
+  :type 'regexp
+  :group 'wl-draft
+  :group 'wl-pref)
+
 (defcustom wl-folder-many-unsync-threshold 70
   "*Folders which contains messages more than this number are highlighted
 with wl-highlight-folder-many-face."
@@ -1949,8 +1991,7 @@ the message is skipped at cursor move."
 (defcustom wl-refile-rule-alist nil
   "Refile rule alist.
 e.x.
-'(
-  (\"From\"
+'((\"From\"
    (\"teranisi@isl.ntt.co.jp\" . \"+teranisi\"))
   (\"x-ml-name\"
    (\"^Wanderlust\"    . \"+wl\")
@@ -2031,6 +2072,11 @@ every intervals specified by wl-biff-check-interval."
   :type 'integer
   :group 'wl-highlight)
 
+(defcustom wl-biff-use-idle-timer nil
+  "Non-nil to use idle timer instead of strict timer for wl-biff"
+  :type 'boolean
+  :group 'wl-highlight)
+
 (defcustom wl-biff-state-indicator-on (if (and (featurep 'xemacs)
 					       (not (featurep 'mule)))
 					  "[Mail]"
@@ -2711,8 +2757,12 @@ This variable can also be a regex."
   :type 'boolean
   :group 'wl-highlight)
 
-(defcustom wl-use-flag-folder-help-echo nil
-  "*Display help-echo in the flag folder if non-nil."
+(defcustom wl-highlight-summary-line-help-echo-alist
+  '((flag wl-highlight-flag-folder-help-echo)
+    (nmz elmo-message-file-name))
+  "*Alist to display help-echo in summary buffer.
+Each element is (folder-type handler(function)).
+Handler take two arguments elmo-folder and message number and return string."
   :type 'boolean
   :group 'wl-highlight)
 
@@ -2724,6 +2774,7 @@ This variable can also be a regex."
   "*Highlight folder with icon(XEmacs or Emacs 21)."
   :type 'boolean
   :group 'wl-highlight)
+
 (defcustom wl-highlight-folder-by-numbers t
   "Highlight folder lines by numbers.
 If it is a number, only numbers will be highlighted."
