@@ -112,7 +112,10 @@
       (progn
 	;; flush queue!!
 	(elmo-dop-queue-flush)
-	(unless queue-flush-only (wl-biff-start))
+	(unless queue-flush-only
+	  (when wl-biff-check-folder-list
+	    (wl-biff-check-folders)
+	    (wl-biff-start)))
 	(if (and wl-draft-enable-queuing
 		 wl-auto-flush-queue)
 	    (wl-draft-queue-flush))
@@ -632,20 +635,29 @@ Entering Plugged mode calls the value of `wl-plugged-mode-hook'."
   (wl-save-status 'keep-summary)
   (run-hooks 'wl-save-hook))
 
+(defun wl-execute-temp-marks ()
+  "Execute temporary marks in summary buffers."
+  (interactive)
+  (let ((summaries (wl-collect-summary)))
+    (while summaries
+      (with-current-buffer (car summaries)
+	(wl-summary-exec-with-confirmation)
+	(wl-summary-save-status))
+      (setq summaries (cdr summaries)))))
+
 (defun wl-save-status (&optional keep-summary)
   (message "Saving summary and folder status...")
-  (let (summary-buf)
-    (save-excursion
-      (let ((summaries (wl-collect-summary)))
-	(while summaries
-	  (with-current-buffer (car summaries)
-	    (unless keep-summary
-	      (wl-summary-cleanup-temp-marks))
-	    (wl-summary-save-view)
-	    (elmo-folder-commit wl-summary-buffer-elmo-folder)
-	    (unless keep-summary
-	      (kill-buffer (car summaries))))
-	  (setq summaries (cdr summaries))))))
+  (save-excursion
+    (let ((summaries (wl-collect-summary)))
+      (while summaries
+	(with-current-buffer (car summaries)
+	  (unless keep-summary
+	    (wl-summary-cleanup-temp-marks))
+	  (wl-summary-save-view)
+	  (elmo-folder-commit wl-summary-buffer-elmo-folder)
+	  (unless keep-summary
+	    (kill-buffer (car summaries))))
+	(setq summaries (cdr summaries)))))
   (wl-refile-alist-save)
   (wl-folder-info-save)
   (and (featurep 'wl-fldmgr) (wl-fldmgr-exit))
@@ -800,7 +812,6 @@ If ARG (prefix argument) is specified, folder checkings are skipped."
   (interactive "P")
   (unless wl-init
     (wl-load-profile)
-    (wl-folder-init)
     (elmo-init))
   (let (demo-buf check)
     (unless wl-init
@@ -819,17 +830,19 @@ If ARG (prefix argument) is specified, folder checkings are skipped."
 		(wl-check-variables-2)
 		(message "Checking type of variables...done")))
 	  (let ((inhibit-quit t))
-	    (wl-plugged-init (wl-folder arg)))
+	    (wl-plugged-init (wl-folder)))
 	  (unless arg
 	    (run-hooks 'wl-auto-check-folder-pre-hook)
 	    (wl-folder-auto-check)
-	    (run-hooks 'wl-auto-check-folder-hook))
-	  (unless arg (wl-biff-start)))
+	    (run-hooks 'wl-auto-check-folder-hook)))
       (error
        (if (buffer-live-p demo-buf)
 	   (kill-buffer demo-buf))
        (signal (car obj)(cdr obj)))
       (quit))
+    (when wl-biff-check-folder-list
+      (unless arg (wl-biff-check-folders))
+      (wl-biff-start))
     (if (buffer-live-p demo-buf)
 	(kill-buffer demo-buf)))
   (run-hooks 'wl-hook))
